"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const AuthorizationValue_1 = __importDefault(require("../models/auth/AuthorizationValue"));
const LoggerFactory_1 = __importDefault(require("../java/LoggerFactory"));
const StringUtils_1 = require("../java/StringUtils");
const StringBuilder_1 = __importDefault(require("../java/StringBuilder"));
const encoders_1 = require("../java/encoders");
class AuthParser {
    static parse(urlEncodedAuthStr) {
        const auths = [];
        if (StringUtils_1.isNotEmpty(urlEncodedAuthStr)) {
            for (const part of urlEncodedAuthStr.split(',')) {
                let kvPair = part.split(':', 2);
                if (kvPair.length === 2) {
                    auths.push(new AuthorizationValue_1.default(encoders_1.URLDecoder.decode(kvPair[0]), encoders_1.URLDecoder.decode(kvPair[1]), 'header'));
                }
            }
        }
        return auths;
    }
    static reconstruct(authorizationValueList) {
        if (authorizationValueList != null) {
            let b = new StringBuilder_1.default();
            for (const v of authorizationValueList) {
                try {
                    if (b.toString().length > 0) {
                        b.append(',');
                    }
                    b.append(encoders_1.URLEncoder.encode(v.getKeyName(), 'UTF-8'))
                        .append(':')
                        .append(encoders_1.URLEncoder.encode(v.getValue(), 'UTF-8'));
                }
                catch (e) {
                    Log.trace(e);
                }
            }
            return b.toString();
        }
        return null;
    }
}
exports.default = AuthParser;
const Log = LoggerFactory_1.default.getLogger(AuthParser);
//# sourceMappingURL=AuthParser.js.map